from libnam import files, control, colors, permissions
from libabr import core
import shutil, requests
import git

## Clean the cache ##
def clean():
    if permissions.check_root(files.readall ("/proc/info/su")):
        if files.isdir ("/app/cache"):
            files.removedirs ("/app/cache")
            files.mkdir("/app/cache")
            files.mkdir("/app/cache/gets")
            files.mkdir("/app/cache/archives")
            files.mkdir("/app/cache/archives/code")
            files.mkdir("/app/cache/archives/control")
            files.mkdir("/app/cache/archives/data")
            files.mkdir("/app/cache/archives/build")
    else:
        colors.show ("libpa.clean","perm","")

## Create .pa archive ##

def build(name):
    if permissions.check_root(files.readall ("/proc/info/su")):
        if not files.isfile (name+"/control/manifest"):
            colors.show ("libpa.build","fail","cannot create archive package")
            clean()
            exit(0)

        if not files.isdir (name+"/data"): files.mkdir(name+"/data")
        if not files.isdir(name + "/code"): files.mkdir(name + "/code")

        shutil.make_archive(files.input("/app/cache/archives/build/data"),"xztar",files.input(name+"/data"))
        shutil.make_archive(files.input("/app/cache/archives/build/code"), "xztar", files.input(name+"/code"))
        shutil.make_archive(files.input("/app/cache/archives/build/control"), "xztar", files.input(name+"/control"))

        shutil.make_archive(files.input(name),"zip",files.input("/app/cache/archives/build"))
        files.cut (name+".zip",name+".pa")
        ## Unlock the cache ##
    else:
        colors.show ("libpa.build","perm","")

## Unpack .pa archives ##

def unpack (name):
    if permissions.check_root(files.readall("/proc/info/su")):
        ## unpack package ##
        shutil.unpack_archive(files.input(name),files.input("/app/cache/archives/build"),"zip")

        shutil.unpack_archive(files.input("/app/cache/archives/build/data.tar.xz"),files.input("/app/cache/archives/data"),"xztar")
        shutil.unpack_archive(files.input("/app/cache/archives/build/code.tar.xz"), files.input("/app/cache/archives/code"), "xztar")
        shutil.unpack_archive(files.input("/app/cache/archives/build/control.tar.xz"), files.input("/app/cache/archives/control"), "xztar")

        ## Get database of this package ##
        name = control.read_record("name", "/app/cache/archives/control/manifest").lower()
        unpack = control.read_record("unpack", "/app/cache/archives/control/manifest")
        depends = control.read_record("depends","/app/cache/archives/control/manifest")

        if not (depends==None):
            depends.split(",")

        ## Search for tree dependency ##

        if not depends==None:
            for i in depends:
                if not files.isfile("/app/packages/" + i + ".manifest"):
                    core.system ('paye -i '+name)

        ## Write dependency ##

        if not depends == None:
            for i in depends:
                files.create("/app/packages/" + i + ".depends")
                files.write("/app/packages/" + i + ".depends", name + "\n")

        ## Run preinstall script ##

        if files.isfile ('/app/cache/archives/control/preinstall.sa'):
            core.system ('saye /app/cache/archives/control/preinstall.sa') # Run it

            ## Copy preinstall script ##

            files.copy ('/app/cache/archives/control/preinstall.sa','/app/packages/'+name+".preinstall")

        ## Setting up ##

        if files.isfile ("/app/cache/archives/control/list"): files.copy("/app/cache/archives/control/list","/app/packages/"+name+".list")
        if files.isfile ("/app/cache/archives/control/manifest"): files.copy("/app/cache/archives/control/manifest","/app/packages/"+name + ".manifest")
        if files.isfile ("/app/cache/archives/control/postinstall.pyc"): files.copy("/app/cache/archives/control/postinstall.pyc","/app/packages/"+ name + ".postinstall")
        if files.isfile ("/app/cache/archives/control/postremove.pyc"): files.copy("/app/cache/archives/control/postremove.pyc","/app/packages/"+ name + ".postremove")
        if files.isfile ("/app/cache/archives/control/preinstall.pyc"): files.copy("/app/cache/archives/control/preinstall.pyc","/app/packages/"+ name + ".preinstall")
        if files.isfile ("/app/cache/archives/control/preremove.pyc"): files.copy("/app/cache/archives/control/preremove.pyc","/app/packages/"+ name + ".preremove")
        if files.isfile ("/app/cache/archives/control/compile"): files.copy("/app/cache/archives/control/compile","/app/packages/"+ name + ".compile")

        ## Compile codes ##
        if files.isfile ("/app/cache/archives/control/compile"):
            listcodes = control.read_list("/app/cache/archives/control/compile")
            for i in listcodes:
                i = i.split(":")
                files.compile("/app/cache/archives/code/" + i[0], "/app/cache/archives/data/" + i[1])

        ## Save source codes ##

        if files.isfile ("/app/cache/archives/control/compile"):
            shutil.unpack_archive(files.input("/app/cache/archives/build/code.tar.xz"),files.input("/usr/src/"+name),"xztar")

        ## Archive data again ##
        shutil.make_archive(files.input("/app/cache/archives/build/data"),"xztar",files.input("/app/cache/archives/data"))

        ## Unpack data again ##
        shutil.unpack_archive(files.input("/app/cache/archives/build/data.tar.xz"),files.input(unpack),"xztar")

        ## After install ##

        ## Run postinstall script ##

        if files.isfile('/app/cache/archives/control/postinstall.sa'):
            core.system('saye /app/cache/archives/control/postinstall.sa')  # Run it

            ## Copy preinstall script ##

            files.copy('/app/cache/archives/control/postinstall.sa', '/app/packages/' + name + ".postinstall")

        ## Copy other scripts ##
        if files.isfile('/app/cache/archives/control/preremove.sa'):
            files.copy ('/app/cache/archives/control/preremove.sa','/app/packages/'+name+".preremove")

        if files.isfile('/app/cache/archives/control/postremove.sa'):
            files.copy ('/app/cache/archives/control/postremove.sa','/app/packages/'+name+".postremove")

        ## Unlock the cache ##
    else:
        colors.show("libpa.unpack", "perm", "")

## Remove package ##
def remove (name):
    if permissions.check_root(files.readall("/proc/info/su")):

        location = "/app/packages/" + name.lower() + ".manifest"

        if not files.isfile(location):
            colors.show("libpa.remove", "fail", name + ": package not found")
            clean()
            exit(0)

        ## Database control ##

        list = "/app/packages/" + name.lower() + ".list"
        preinstall = "/app/packages/"+name.lower()+".preinstall"
        postinstall = "/app/packages/"+name.lower()+".postinstall"
        preremove = "/app/packages/"+name.lower()+".preremove"
        postremove = "/app/packages/"+name.lower()+".postremove"
        compile = "/app/packages/"+name.lower()+".compile"
        depends = "/app/packages/"+name.lower()+".depends"


        ## Create preremove and postremove copies ##

        if files.isfile(preremove): files.copy(preremove,"/app/cache/archives/control/preremove.sa")
        if files.isfile(postremove): files.copy(postremove, "/app/cache/archives/control/postremove.sa")

        ## Run pre remove script ##

        if files.isfile("/app/cache/archives/control/preremove.sa"):
            core.system("saye /app/cache/archives/control/preremove.sa")

        ## Remove depends ##

        if files.isfile(depends):
            depends = control.read_list(depends)
            for i in depends:
                remove(i)

        ####################

        unpack = control.read_record("unpack",location)

        ## Unpacked removal ##
        filelist = control.read_list(list)

        for i in filelist:
            if files.isdir (unpack+"/"+i):
                files.removedirs(unpack + "/" + i)
            elif files.isfile (unpack+"/"+i):
                files.remove (unpack+"/"+i)

        ## Database removal ##

        if files.isfile (location): files.remove (location)
        if files.isfile(compile): files.remove(compile)
        if files.isfile(list): files.remove(list)
        if files.isfile(preinstall): files.remove(preinstall)
        if files.isfile(postinstall): files.remove(postinstall)
        if files.isfile(preremove): files.remove(preremove)
        if files.isfile(postremove): files.remove(postremove)
        if files.isfile (depends): files.remove(depends)

        ## Remove source code ##
        if files.isdir ('/usr/src/'+name): files.removedirs('/usr/src/'+name)

        ## Run postremove script ##

        if files.isfile("/app/cache/archives/control/postremove.sa"):
            core.system("saye /app/cache/archives/control/postremove.sa")

    else:
        colors.show ("libpa.remove","perm","")

## Download package ##
def download (packname):
    if permissions.check_root(files.readall("/proc/info/su")):
        mirror = control.read_record("mirror","/etc/repo")
        git.Git(files.input("/app/cache/gets/")).clone(mirror+"/"+packname) # Git source code package # Helped from stockoverflow
        build('/app/cache/gets/'+packname) # Pack the git source
        files.removedirs ('/app/cache/gets/'+packname) # Remove git source after packing
    else:
        colors.show ("libpa.download","perm","")